"""SNAPtoolbelt Serial Flash Commands"""
import logging
from textwrap import dedent
from argparse import RawDescriptionHelpFormatter

from arghandler import ArgumentHandler
import progressbar

import snapstraction.flash

log = logging.getLogger(__name__)


def serial(parser, context, args):
    """Serial Flasher commands"""
    handler = ArgumentHandler(
        usage="snap [global options] serial [-h] (default-nvparam|erase-script|flash-firmware|flash-sniffer) [options]",
        description=dedent("""
        SNAP Serial Commands
          default-nvparam: Default NV Params on the serial bridge node
          erase-script: Erase the script on the serial bridge node
          flash-firmware: Flash SNAP Firmware on the serial bridge node
          flash-sniffer: Flash SNAP Sniffer Firmware on the serial bridge node
        """),
        formatter_class=RawDescriptionHelpFormatter,
    )

    # Add subcommands here.
    handler.set_subcommands({
        'default-nvparam': default_nv,
        'erase-script': erase,
        'flash-firmware': firmware,
        'flash-sniffer': sniffer,
    })

    if context.profile.kind != 'serial':
        log.error(
            "Serial commands only work with direct serial access, %s not supported.",
            context.profile['snap_bridge_type'])
        exit(1)

    handler.run(args, context_fxn=lambda x: context)


def firmware(parser, context, args):  # noqa
    """Upload firmware via serial"""
    handler = ArgumentHandler(
        usage="snap [global options] serial flash-firmware [-h] <module> <core> [--file <file>]",
        description="Flash the SNAP firmware on the bridge node",
    )

    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")
    handler.add_argument('core', help="SNAP Core version (i.e. '2.6.9')")
    handler.add_argument('-f', '--file', default=None, help="Override firmware file path")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected arguments to serial flash-firmware command: %r", rest)
        exit(1)

    log.info("Loading firmware for module %s, core %s", args.module, args.core)

    widgets = [progressbar.FormatLabel('Flash Firmware:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        """Update Progress Bar"""
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        """Finish Progress Bar"""
        progress.finish()

    progress.start()

    snapstraction.flash.firmware(context.profile, args.module, args.core, sniffer=False, override_file=args.file,
                                 progress_cb=on_progress, finished_cb=on_finished)


def sniffer(parser, context, args):  # noqa
    """Upload firmware via serial"""
    handler = ArgumentHandler(
        usage="snap [global options] serial flash-sniffer [-h] <module> [--file <file>]",
        description="Flash the Sniffer firmware on the bridge node",
    )

    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")
    handler.add_argument('-f', '--file', default=None, help="Override firmware file path")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected arguments to serial flash-sniffer command: %r", rest)
        exit(1)

    log.info("Loading sniffer firmware for module %s", args.module)

    widgets = [progressbar.FormatLabel('Flash Sniffer:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        """Update Progress Bar"""
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        """Finish Progress Bar"""
        progress.finish()

    progress.start()

    snapstraction.flash.firmware(context.profile, args.module, '_', sniffer=True, override_file=args.file,
                                 progress_cb=on_progress, finished_cb=on_finished)


def default_nv(parser, context, args):
    """Reset NV Params via serial"""
    handler = ArgumentHandler(
        usage="snap [global options] serial default-nvparam [-h] <module>",
        description="Default the NV Parameters on the bridge node",
    )

    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected arguments to serial default-nvparam command: %r", rest)
        exit(1)

    log.info("Defaulting NV Params for module %s", args.module)

    widgets = [progressbar.FormatLabel('Default NV Params:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        """Update Progress Bar"""
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        """Finish Progress Bar"""
        progress.finish()

    progress.start()

    snapstraction.flash.default_nv(context.profile,
                                   args.module,
                                   progress_cb=on_progress,
                                   finished_cb=on_finished)


def erase(parser, context, args):
    """Erase Script via serial"""
    handler = ArgumentHandler(
        usage="snap [global options] serial erase-script [-h] <module>",
        description="Erase the script on the bridge node",
    )

    handler.add_argument('module',
                         help="Module name (i.e. 'RF220UF1', 'SS200')")

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected arguments to serial erase-script command: %r", rest)
        exit(1)

    log.info("Erasing SNAPpy Image for module %s", args.module)

    widgets = [progressbar.FormatLabel('Erase Script:'), ' ',
               progressbar.Percentage(), ' ', progressbar.Bar(), ' ',
               progressbar.Timer(), ' ', progressbar.AdaptiveETA()]

    progress = progressbar.ProgressBar(widgets=widgets, maxval=1000)

    def on_progress(counter, new_max):
        """Update Progress Bar"""
        progress.maxval = max(new_max, counter)
        progress.update(counter)

    def on_finished():
        """Finish Progress Bar"""
        progress.finish()

    progress.start()

    snapstraction.flash.erase_script(context.profile,
                                     args.module,
                                     progress_cb=on_progress,
                                     finished_cb=on_finished)
